/*
	Vibrator Control Protocol Daemon
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

#include <ser.h>
#include <vcp.h>

int ser_init( S_VCP_STATE *vs , char *port )
{
	unsigned char buffer[5];
	int errors;

	vs->func.vibr=vcp_vibr_dummy;
	vs->func.intv=vcp_intv_dummy;

	vs->si = SIO_Open( port );
	if( vs->si==NULL ) {
		syslog( LOG_ERR, "Unable to open device.\n" );
		return FALSE;
	}

	vs->func.vibr=ser_vibr;
	vs->func.intv=ser_intv;

	/* Set main port parameters */
	SIO_SetSpeed( vs->si, 9600 );
	SIO_SetDataBits( vs->si, 8 );
	SIO_SetParity( vs->si, SIO_PARITY_NONE );
	SIO_SetStopBits( vs->si, 1 );
	SIO_SetIOMode( vs->si, SIO_IOMODE_DIRECT );
	SIO_WriteSettings( vs->si );

	for( errors=0; errors<3; errors++ ) {
		/* Get unwanted bytes */
		SIO_SetReadTimeout( vs->si, 100000 );
		while( SIO_ReadChar( vs->si )!=-1 ) {}

		SIO_SetReadTimeout( vs->si, 1000000 );

		/* Reset device */
		usleep( 1000 );
		SIO_RaiseDTR( vs->si ); /* +12V */
		usleep( 1000 );
		SIO_DropDTR( vs->si ); /* -12V */

		if( SIO_ReadBuffer( vs->si, buffer, 5 )!=5 ) continue;

		if( memcmp( buffer, "BHEF1", 5 )!=0 ) continue;

		syslog( LOG_INFO, "Vibrator detected.\n" );
		return TRUE;
	}

	syslog( LOG_ERR, "Vibrator not responding.\n" );

	return FALSE;
}

int	ser_vibr( S_VCP_STATE *vs, int level )
{
	int c;

	if( level>255 ) level=255;

	/* Send CMD_SET */
	if( SIO_WriteChar( vs->si, CMD_SET )==-1 ) return FALSE;
	if( ((c=SIO_ReadChar( vs->si ))==-1) || (c!=CMD_SET) ) return FALSE;

	/* Send higher byte of level */
	if( SIO_WriteChar( vs->si, level )==-1 ) return FALSE;
	if( ((c=SIO_ReadChar( vs->si ))==-1) || (c!=level) ) return FALSE;

	vs->vibr=level;

	return TRUE;
}

int	ser_intv( S_VCP_STATE *vs, char *seq )
{
	return FALSE;
}

