/*
	Vibrator Control Protocol Daemon
	Copyright (C) 2003  Matthias Bruestle <m@mbsks.franken.de>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef VCP_H
#define VCP_H

#include <sio.h>

#define VCP_VERSION	"1.0"
#define VCP_PORT	6900
#define VCP_BUFLEN	1024
#define VCP_CONFIG	"/etc/vcpdrc"

#define VCP_HELP "Available commands:\nAUTH <user> <password>\nHELP\nINFO\nQUIT\nVIBR <level 0-255>\n"

#define VCP_RETCODE_STATUS	"211 Status\n"
#define VCP_RETCODE_HELP	"214 Help\n"
#define VCP_RETCODE_READY	"220 VCP %s\n"
#define VCP_RETCODE_CLOSE	"221 Bye\n"
#define VCP_RETCODE_OK		"250 Ok\n"
#define VCP_RETCODE_UNAUTH	"401 Unauthorized\n"
#define VCP_RETCODE_BUSY	"421 Service already in use\n"
#define VCP_RETCODE_ERROR	"451 Error\n"
#define VCP_RETCODE_UNKN	"500 Unknown command\n"
#define VCP_RETCODE_SYNTAX	"501 Syntax error in parameters\n"
#define VCP_RETCODE_CNIMPL	"502 Command not implemented\n"
#define VCP_RETCODE_PNIMPL	"504 Parameter not implemented\n"

#ifndef FALSE
#define FALSE   0
#endif
#ifndef TRUE
#define TRUE    (!FALSE)
#endif

typedef enum {
	VIB_TYPE_SER,
	VIB_TYPE_DUMMY,
	VIB_TYPE_LAST
} vib_type_t;

typedef struct vcp_state {
	vib_type_t	type;
	int			vibr;
	int			cs;
	SIO_INFO	*si;
	char		buffer[VCP_BUFLEN+1];
	char		hostname[80];
	char		username[80];
	struct		s_auth {
		int			verified;
		char		login[20];
		char		password[20];
	} auth;
	struct		s_func {
		int			(* vibr)( struct vcp_state *vs, int level );
		int			(* intv)( struct vcp_state *vs, char *seq );
	} func;
} S_VCP_STATE;

void vcp_dump_state( S_VCP_STATE *vs );
int vcp_init( S_VCP_STATE *vs, char *port );
int vcp_control( S_VCP_STATE *vs );
int vcp_vibr_dummy( S_VCP_STATE *vs, int level );
int vcp_intv_dummy( S_VCP_STATE *vs, char *seq );

#endif

